
//  Author:  Chuck Stewart
//  File:    Complex.h 
//  Purpose: The declaration for a complex number class.

#include <iostream>

using std::istream;
using std::ostream;

class Complex {
public:
  //  Constructor with default arguments
  Complex( double x=0, double y=0 );

  //  Copy constructor; copies an existing Complex object
  Complex( Complex const& old );

  //  Assignment operator
  Complex& operator= ( Complex const& rhs );

  //  "Ordinary" member functions, including accessors and mutators
 
  double Real() const { return real_; }
  void SetReal( double x ) { real_ = x; }
  double Imaginary() const { return imag_; }
  void SetImaginary( double y ) { imag_ = y; }
  double Magnitude() const;

  //  operator+ as a member function.
  Complex operator+ ( Complex const& rhs ) const;

  //  unary operator- to create the negation of a complex number 
  Complex operator- () const;

  //  Input and output stream operators can not be member functions.
  //  They can be friends, or they can be non-member functions if
  //  their work can be accomplished through the public interface to
  //  the class.  Note that the complex object passed to the
  //  istream (>>) operator MUST be passed as a non-const reference.

  friend istream& operator>> ( istream& istr, Complex& c );

private:
  //  Note again the _ at the end of the member variable names.  This
  //  convention is often used to easily distinguish member variables
  //  in the middle of member function code.
  double real_, imag_;
};


//  The following two prototypes are for ordinary non-member functions.

Complex operator- ( Complex const& left, Complex const& right );
ostream& operator<< ( ostream& ostr, Complex const& c );

